# -*- coding: utf-8 -*-

import types
from django import template
from django.template.loader import get_template
from django.template import Context
from django.forms import CheckboxInput
register = template.Library()


@register.filter
def as_foundation(form):
    template = get_template("foundation_formtags/form.html")
    c = {"form": form}
    return template.render(c)


@register.inclusion_tag('foundation_formtags/foundation_form_field.html')
def render_field(field):
    """
    Use this need tag to get more control over the layout of your forms
    {% raw %}{% render_field form.my_field %} {% endraw %}
    """
    return {'field': field}


@register.filter(name='is_checkbox')
def is_checkbox(field):
    return field.field.widget.__class__.__name__ == CheckboxInput(
    ).__class__.__name__


def silence_without_field(fn):
    def wrapped(field, attr):
        if not field:
            return ""
        return fn(field, attr)
    return wrapped


def _process_field_attributes(field, attr, process):

    # split attribute name and value from 'attr:value' string
    params = attr.split(':', 1)
    attribute = params[0]
    value = params[1] if len(params) == 2 else ''

    # decorate field.as_widget method with updated attributes
    old_as_widget = field.as_widget

    def as_widget(self, widget=None, attrs=None, only_initial=False):
        attrs = attrs or {}
        process(widget or self.field.widget, attrs, attribute, value)
        html = old_as_widget(widget, attrs, only_initial)
        self.as_widget = old_as_widget
        return html

    field.as_widget = types.MethodType(as_widget, field)
    return field


@register.filter("append_attr")
@silence_without_field
def append_attr(field, attr):
    def process(widget, attrs, attribute, value):
        if attrs.get(attribute):
            attrs[attribute] += ' ' + value
        elif widget.attrs.get(attribute):
            attrs[attribute] = widget.attrs[attribute] + ' ' + value
        else:
            attrs[attribute] = value
    return _process_field_attributes(field, attr, process)


@register.filter("add_class")
@silence_without_field
def add_class(field, css_class):
    return append_attr(field, 'class:' + css_class)


@register.filter("add_error_class")
@silence_without_field
def add_error_class(field, css_class):
    if hasattr(field, 'errors') and field.errors:
        return add_class(field, css_class)
    return field



from django import forms

CHOICES = (
    ('a', 'Option 1'),
    ('b', 'Option 2'),
    ('c', 'Option 3'),
)


class SimpleForm(forms.Form):
    text = forms.CharField(label='text')


class ComplexForm(forms.Form):
    char_field = forms.CharField()
    char_field_widget = forms.CharField(widget=forms.TextInput(attrs={'class':'foo'}))
    choice_field = forms.ChoiceField(choices=CHOICES)
    radio_choice = forms.ChoiceField(choices=CHOICES, widget=forms.RadioSelect)
    multiple_choice = forms.MultipleChoiceField(choices=CHOICES)
    multiple_checkbox = forms.MultipleChoiceField(
        choices=CHOICES, widget=forms.CheckboxSelectMultiple
    )
    file_field = forms.FileField()
    password_field = forms.CharField(widget=forms.PasswordInput)
    textarea = forms.CharField(widget=forms.Textarea)
    boolean_field = forms.BooleanField()



#from django.template import Context
from django.template import engines
import unittest

from .forms import SimpleForm, ComplexForm


def render_form(text, form=None, **context_args):
    tpl = engines['django'].from_string("{% load foundation_formtags %}" + text)
    context_args.update({'form': ComplexForm() if form is None else form})
    #context = Context(context_args)
    return tpl.render(context_args)


def render_field(field, template_filter, params, *args, **kwargs):
    filters = [(template_filter, params)]
    filters.extend(zip(args[::2], args[1::2]))
    filter_strings = ['|%s:"%s"' % (f[0], f[1],) for f in filters]
    render_field_str = '{{ form.%s%s }}' % (field, ''.join(filter_strings))
    return render_form(render_field_str, **kwargs)


class TestFoundationform(unittest.TestCase):

    def filter_or_tag_test(self, template, context, contains):
        tpl = engines['django'].from_string(
            "{% load foundation_formtags %}" + template
        )
        self.assertIn(contains, tpl.render(context))

    def test_as_foundation_filter_simple(self):
        data = {'text': 'Tests'}
        simple_form = SimpleForm(data)
        template = '{{ form|as_foundation }}'
        context = {'form': simple_form}
        contains = 'name="text"'

        self.filter_or_tag_test(template, context, contains)

    def test_as_foundation_filter_complex(self):
        data = {'char_field': 'Tests', 'password_field': 'password',
                'choice_field': 'Option 1', 'boolean_field': 'True'}
        complex_form = ComplexForm(data)
        template = '{{ form|as_foundation }}'
        context = {'form': complex_form}
        contains = 'type="password"'

        self.filter_or_tag_test(template, context, contains)

    def test_render_field_tag_simple(self):
        data = {'text': 'Tests'}
        simple_form = SimpleForm(data)
        template = '<form class="form" action="" method="POST"> \
                        {% csrf_token %} \
                        {% render_field form.text %} \
                    </form>'
        context = {'form': simple_form}
        contains = 'id="id_text"'

        self.filter_or_tag_test(template, context, contains)

    def test_render_field_tag_complex(self):
        #import ipdb; ipdb.set_trace() ### BREAK-POINT
        data = {'char_field': 'Tests', 'password_field': 'password',
                'choice_field': 'Option 1', 'boolean_field': 'True'}
        complex_form = ComplexForm(data)
        template = '<form class="form" action="" method="POST"> \
                        {% csrf_token %} \
                        {% render_field form.char_field %} \
                        {% render_field form.password_field %} \
                    </form>'
        context = {'form': complex_form}
        contains = 'id="id_password_field"'

        self.filter_or_tag_test(template, context, contains)

    def test_add_multiple_classes(self):
        res = render_field('char_field', 'add_class', 'foo bar')
        self.assertIn('class="foo bar"', res)

    def test_add_class_chaining(self):
        res = render_field('char_field', 'add_class', 'foo', 'add_class', 'bar')
        self.assertIn('class="bar foo"', res)

    def test_silence_without_field(self):
        res = render_field("nothing", 'add_class', 'some')
        self.assertEqual(res, "")

if __name__ == '__main__':
    unittest.main()
